import { __assign, __decorate, __param } from 'tslib';
import { EventEmitter, Optional, Inject, Input, Output, HostListener, Component, ElementRef, Directive, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { DomSanitizer } from '@angular/platform-browser';

var CustomImageEvent = /** @class */ (function () {
    function CustomImageEvent(name, imageIndex) {
        this.name = name;
        this.imageIndex = imageIndex;
    }
    return CustomImageEvent;
}());

var DEFAULT_CONFIG = {
    btnClass: 'default',
    zoomFactor: 0.1,
    containerBackgroundColor: '#ccc',
    wheelZoom: false,
    allowFullscreen: true,
    allowKeyboardNavigation: true,
    btnShow: {
        zoomIn: true,
        zoomOut: true,
        rotateClockwise: true,
        rotateCounterClockwise: true,
        next: true,
        prev: true
    },
    btnIcons: {
        zoomIn: 'fa fa-plus',
        zoomOut: 'fa fa-minus',
        rotateClockwise: 'fa fa-repeat',
        rotateCounterClockwise: 'fa fa-undo',
        next: 'fa fa-arrow-right',
        prev: 'fa fa-arrow-left',
        fullscreen: 'fa fa-arrows-alt',
    }
};
var AngularImageViewerComponent = /** @class */ (function () {
    function AngularImageViewerComponent(moduleConfig, sanitizer) {
        this.moduleConfig = moduleConfig;
        this.sanitizer = sanitizer;
        this.index = 0;
        this.indexChange = new EventEmitter();
        this.configChange = new EventEmitter();
        this.customImageEvent = new EventEmitter();
        this.styleHeight = '98vh';
        this.style = { transform: '', msTransform: '', oTransform: '', webkitTransform: '' };
        this.fullscreen = false;
        this.loading = true;
        this.scale = 1;
        this.rotation = 0;
        this.translateX = 0;
        this.translateY = 0;
        this.hovered = false;
    }
    AngularImageViewerComponent.prototype.ngOnChanges = function (changes) {
        if (changes.screenHeightOccupied) {
            this.styleHeight = 'calc(98vh - ' + this.screenHeightOccupied + 'px)';
            // console.log('Style Height:', this.styleHeight);
        }
    };
    AngularImageViewerComponent.prototype.ngOnInit = function () {
        var merged = this.mergeConfig(DEFAULT_CONFIG, this.moduleConfig);
        this.config = this.mergeConfig(merged, this.config);
        this.triggerConfigBinding();
    };
    AngularImageViewerComponent.prototype.nextImage = function (event) {
        if (this.canNavigate(event) && this.index < this.src.length - 1) {
            this.loading = true;
            this.index++;
            this.triggerIndexBinding();
            this.reset();
        }
    };
    AngularImageViewerComponent.prototype.prevImage = function (event) {
        if (this.canNavigate(event) && this.index > 0) {
            this.loading = true;
            this.index--;
            this.triggerIndexBinding();
            this.reset();
        }
    };
    AngularImageViewerComponent.prototype.zoomIn = function () {
        this.scale *= (1 + this.config.zoomFactor);
        this.updateStyle();
    };
    AngularImageViewerComponent.prototype.zoomOut = function () {
        if (this.scale > this.config.zoomFactor) {
            this.scale /= (1 + this.config.zoomFactor);
        }
        this.updateStyle();
    };
    AngularImageViewerComponent.prototype.scrollZoom = function (evt) {
        if (this.config.wheelZoom) {
            evt.deltaY > 0 ? this.zoomOut() : this.zoomIn();
            return false;
        }
    };
    AngularImageViewerComponent.prototype.rotateClockwise = function () {
        this.rotation += 90;
        this.updateStyle();
    };
    AngularImageViewerComponent.prototype.rotateCounterClockwise = function () {
        this.rotation -= 90;
        this.updateStyle();
    };
    AngularImageViewerComponent.prototype.onLoad = function (url) {
        console.log('Loading Image Done:', url);
        this.loading = false;
    };
    AngularImageViewerComponent.prototype.onLoadStart = function (url) {
        console.log('Loading Image:', url);
        this.loading = true;
    };
    AngularImageViewerComponent.prototype.imageNotFound = function (url) {
        console.log('Image not found Url:', url);
    };
    AngularImageViewerComponent.prototype.onDragOver = function (evt) {
        this.translateX += (evt.clientX - this.prevX);
        this.translateY += (evt.clientY - this.prevY);
        this.prevX = evt.clientX;
        this.prevY = evt.clientY;
        this.updateStyle();
    };
    AngularImageViewerComponent.prototype.onDragStart = function (evt) {
        if (evt.dataTransfer && evt.dataTransfer.setDragImage) {
            evt.dataTransfer.setDragImage(evt.target.nextElementSibling, 0, 0);
        }
        this.prevX = evt.clientX;
        this.prevY = evt.clientY;
    };
    AngularImageViewerComponent.prototype.toggleFullscreen = function () {
        this.fullscreen = !this.fullscreen;
        if (!this.fullscreen) {
            this.reset();
        }
    };
    AngularImageViewerComponent.prototype.triggerIndexBinding = function () {
        this.indexChange.emit(this.index);
    };
    AngularImageViewerComponent.prototype.triggerConfigBinding = function () {
        this.configChange.next(this.config);
    };
    AngularImageViewerComponent.prototype.fireCustomEvent = function (name, imageIndex) {
        this.customImageEvent.emit(new CustomImageEvent(name, imageIndex));
    };
    AngularImageViewerComponent.prototype.reset = function () {
        this.scale = 1;
        this.rotation = 0;
        this.translateX = 0;
        this.translateY = 0;
        this.updateStyle();
    };
    AngularImageViewerComponent.prototype.onMouseOver = function () {
        this.hovered = true;
    };
    AngularImageViewerComponent.prototype.onMouseLeave = function () {
        this.hovered = false;
    };
    AngularImageViewerComponent.prototype.canNavigate = function (event) {
        return event == null || (this.config.allowKeyboardNavigation && this.hovered);
    };
    AngularImageViewerComponent.prototype.updateStyle = function () {
        this.style.transform = "translate(" + this.translateX + "px, " + this.translateY + "px) rotate(" + this.rotation + "deg) scale(" + this.scale + ")";
        this.style.msTransform = this.style.transform;
        this.style.webkitTransform = this.style.transform;
        this.style.oTransform = this.style.transform;
    };
    AngularImageViewerComponent.prototype.mergeConfig = function (defaultValues, overrideValues) {
        var result = __assign({}, defaultValues);
        if (overrideValues) {
            result = __assign(__assign({}, defaultValues), overrideValues);
            if (overrideValues.btnIcons) {
                result.btnIcons = __assign(__assign({}, defaultValues.btnIcons), overrideValues.btnIcons);
            }
        }
        return result;
    };
    AngularImageViewerComponent.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: ['config',] }] },
        { type: DomSanitizer }
    ]; };
    __decorate([
        Input()
    ], AngularImageViewerComponent.prototype, "src", void 0);
    __decorate([
        Input()
    ], AngularImageViewerComponent.prototype, "screenHeightOccupied", void 0);
    __decorate([
        Input()
    ], AngularImageViewerComponent.prototype, "index", void 0);
    __decorate([
        Input()
    ], AngularImageViewerComponent.prototype, "config", void 0);
    __decorate([
        Output()
    ], AngularImageViewerComponent.prototype, "indexChange", void 0);
    __decorate([
        Output()
    ], AngularImageViewerComponent.prototype, "configChange", void 0);
    __decorate([
        Output()
    ], AngularImageViewerComponent.prototype, "customImageEvent", void 0);
    __decorate([
        HostListener('window:keyup.ArrowRight', ['$event'])
    ], AngularImageViewerComponent.prototype, "nextImage", null);
    __decorate([
        HostListener('window:keyup.ArrowLeft', ['$event'])
    ], AngularImageViewerComponent.prototype, "prevImage", null);
    __decorate([
        HostListener('mouseover')
    ], AngularImageViewerComponent.prototype, "onMouseOver", null);
    __decorate([
        HostListener('mouseleave')
    ], AngularImageViewerComponent.prototype, "onMouseLeave", null);
    AngularImageViewerComponent = __decorate([
        Component({
            selector: 'angular-image-viewer',
            template: "<div [appScreenfull]=\"fullscreen\" class=\"img-container\" [style.height]=\"styleHeight\"\n    [style.backgroundColor]=\"config.containerBackgroundColor\" (wheel)=\"scrollZoom($event)\"\n    (dragover)=\"onDragOver($event)\">\n    <img [src]=\"src[index]\" [ngStyle]=\"style\" alt=\"Image not found...\" (dragstart)=\"onDragStart($event)\"\n        (load)=\"onLoad(src[index])\" (error)=\"imageNotFound(src[index])\" (loadstart)=\"onLoadStart(src[index])\" />\n    <!-- Div below will be used to hide the 'ghost' image when dragging -->\n    <div></div>\n    <div class=\"spinner-container\" *ngIf=\"loading\">\n        <div class=\"spinner\"></div>\n    </div>\n\n    <button type=\"button\" [class]=\"config.btnClass\" *ngIf=\"config.btnShow.rotateCounterClockwise\"\n        (click)=\"rotateCounterClockwise()\">\n        <span [class]=\"config.btnIcons.rotateCounterClockwise\"></span>\n    </button>\n    <button type=\"button\" [class]=\"config.btnClass\" *ngIf=\"config.btnShow.rotateClockwise\" (click)=\"rotateClockwise()\">\n        <span [class]=\"config.btnIcons.rotateClockwise\"></span>\n    </button>\n\n    <button type=\"button\" [class]=\"config.btnClass\" *ngIf=\"config.btnShow.zoomOut\" (click)=\"zoomOut()\">\n        <span [class]=\"config.btnIcons.zoomOut\"></span>\n    </button>\n    <button type=\"button\" [class]=\"config.btnClass\" *ngIf=\"config.btnShow.zoomIn\" (click)=\"zoomIn()\">\n        <span [class]=\"config.btnIcons.zoomIn\"></span>\n    </button>\n\n    <button type=\"button\" [class]=\"config.btnClass\" *ngFor=\"let cBtn of config.customBtns\"\n        (click)=\"fireCustomEvent(cBtn.name, index)\">\n        <span [class]=\"cBtn.icon\"></span>\n    </button>\n\n    <button type=\"button\" id=\"ngx-fs-btn\" [class]=\"config.btnClass\" (click)=\"toggleFullscreen()\"\n        *ngIf=\"config.allowFullscreen\">\n        <span [class]=\"config.btnIcons.fullscreen\"></span>\n    </button>\n\n    <div class=\"nav-button-container\" *ngIf=\"src.length > 1\">\n        <button type=\"button\" [class]=\"config.btnClass\" (click)=\"prevImage($event)\" [disabled]=\"index === 0\">\n            <span [class]=\"config.btnIcons.prev\"></span>\n        </button>\n        <button type=\"button\" [class]=\"config.btnClass\" (click)=\"nextImage($event)\"\n            [disabled]=\"index === src.length - 1\">\n            <span [class]=\"config.btnIcons.next\"></span>\n        </button>\n    </div>\n</div>",
            styles: [".img-container{width:100%;display:-webkit-box;display:flex;-webkit-box-pack:center;justify-content:center;-webkit-box-align:center;align-items:center;overflow:hidden}.img-container img{max-width:90%;max-height:95%}.img-container button{z-index:99;position:absolute;right:15px}.img-container button:not(:disabled){cursor:pointer}.img-container>button:nth-of-type(1):not(#ngx-fs-btn){bottom:15px}.img-container>button:nth-of-type(2):not(#ngx-fs-btn){bottom:65px}.img-container>button:nth-of-type(3):not(#ngx-fs-btn){bottom:115px}.img-container>button:nth-of-type(4):not(#ngx-fs-btn){bottom:165px}.img-container>button:nth-of-type(5):not(#ngx-fs-btn){bottom:215px}.img-container>button:nth-of-type(6):not(#ngx-fs-btn){bottom:265px}.img-container>button:nth-of-type(7):not(#ngx-fs-btn){bottom:315px}#ngx-fs-btn{top:15px}button.default{height:40px;width:40px;border:1px solid #555;border-radius:50%;background-color:#fff;opacity:.7;-webkit-transition:opacity .2s;transition:opacity .2s}button.default:hover{opacity:1}button.default:disabled{opacity:.25}.nav-button-container>button{position:relative;right:0;margin:0 10px}.nav-button-container{text-align:center;position:absolute;z-index:98;bottom:10px;left:0;right:0}.spinner-container{position:absolute;left:0;right:0;top:0;bottom:0;width:60px;height:60px;margin:auto;padding:10px;background-color:rgba(0,0,0,.4);border-radius:25%}.spinner{border-width:7px;border-style:solid;border-color:#ccc #ccc #222;border-radius:50%;height:100%;width:100%;box-sizing:border-box;-webkit-animation:2s linear infinite rotation;animation:2s linear infinite rotation}@keyframes rotation{from{-webkit-transform:rotate(0)}to{-webkit-transform:rotate(359deg)}}@-webkit-keyframes rotation{from{-webkit-transform:rotate(0)}to{-webkit-transform:rotate(359deg)}}"]
        }),
        __param(0, Optional()), __param(0, Inject('config'))
    ], AngularImageViewerComponent);
    return AngularImageViewerComponent;
}());

// import * as screenfull from 'screenfull';
var FullScreenDirective = /** @class */ (function () {
    function FullScreenDirective(el) {
        this.el = el;
    }
    FullScreenDirective.prototype.ngOnChanges = function (changes) {
        console.log('fullscreenState isFirstChange:', changes["fullscreenState"].isFirstChange());
        console.log('fullscreenState', this.fullscreenState);
        // if (screenfull.isEnabled) {
        //   screenfull.toggle(this.el.nativeElement);
        // }
        // if (this.fullscreenState && screenfull.isEnabled) {
        //   screenfull.request(this.el.nativeElement);
        // } else if (screenfull.isEnabled) {
        //   screenfull.exit();
        // }
        // tslint:disable-next-line: no-string-literal
        if (!changes['fullscreenState'].isFirstChange()) {
            if (this.fullscreenState) {
                var element = this.el.nativeElement;
                // tslint:disable-next-line: max-line-length
                var requestMethod = element.requestFullscreen || element.webkitRequestFullScreen || element.mozRequestFullScreen || element.msRequestFullScreen;
                if (requestMethod) { // Native full screen.
                    requestMethod.call(element);
                }
                else {
                    console.log('FullScreen Request Method Not Supported on this browser.');
                }
            }
            else {
                var element = document;
                // tslint:disable-next-line: max-line-length
                var requestMethod = element.cancelFullscreen || element.webkitExitFullscreen || element.webkitCancelFullScreen || element.mozCancelFullScreen || element.msExitFullScreen;
                if (requestMethod) { // Native Cancel full screen.
                    requestMethod.call(element);
                }
                else {
                    console.log('FullScreen Cancel Request Method Not Supported on this browser.');
                }
            }
        }
    };
    FullScreenDirective.prototype.ngOnInit = function () {
    };
    FullScreenDirective.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    __decorate([
        Input('appScreenfull')
    ], FullScreenDirective.prototype, "fullscreenState", void 0);
    FullScreenDirective = __decorate([
        Directive({
            selector: '[appScreenfull]'
        })
    ], FullScreenDirective);
    return FullScreenDirective;
}());

// import { ImageViewerConfig } from 'dist/angular-image-viewer/public-api';
var AngularImageViewerModule = /** @class */ (function () {
    function AngularImageViewerModule() {
    }
    AngularImageViewerModule = __decorate([
        NgModule({
            declarations: [AngularImageViewerComponent, FullScreenDirective],
            imports: [
                CommonModule,
            ],
            exports: [AngularImageViewerComponent, FullScreenDirective]
        })
    ], AngularImageViewerModule);
    return AngularImageViewerModule;
}());

/*
 * Public API Surface of angular-image-viewer
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AngularImageViewerComponent, AngularImageViewerModule, CustomImageEvent, FullScreenDirective };
//# sourceMappingURL=angular-x-image-viewer.js.map
